@@ -556,17 +556,24 @@ void MapblockMeshGenerator::drawLiquidSides()
 		for (int j = 0; j < 4; j++) {
 			const UV &vertex = base_vertices[j];
 			const v3s16 &base = face.p[vertex.u];
+			float v = vertex.v;
+
 			v3f pos;
-			pos.X = (base.X - 0.5) * BS;
-			pos.Z = (base.Z - 0.5) * BS;
-			if (vertex.v)
-				pos.Y = neighbor.is_same_liquid ? corner_levels[base.Z][base.X] : -0.5 * BS;
-			else
-				pos.Y =     !top_is_same_liquid ? corner_levels[base.Z][base.X] :  0.5 * BS;
+			pos.X = (base.X - 0.5f) * BS;
+			pos.Z = (base.Z - 0.5f) * BS;
+			if (vertex.v) {
+				pos.Y = neighbor.is_same_liquid ? corner_levels[base.Z][base.X] : -0.5f * BS;
+			} else if (top_is_same_liquid) {
+				pos.Y = 0.5f * BS;
+			} else {
+				pos.Y = corner_levels[base.Z][base.X];
+				v += (0.5f * BS - corner_levels[base.Z][base.X]) / BS;
+			}
+
 			if (data->m_smooth_lighting)
 				color = blendLightColor(pos);
 			pos += origin;
-			vertices[j] = video::S3DVertex(pos.X, pos.Y, pos.Z, 0, 0, 0, color, vertex.u, vertex.v);
+			vertices[j] = video::S3DVertex(pos.X, pos.Y, pos.Z, 0, 0, 0, color, vertex.u, v);
 		};
 		collector->append(tile_liquid, vertices, 4, quad_indices, 6);
 	}
@@ -860,17 +867,27 @@ void MapblockMeshGenerator::drawTorchlikeNode()
 	for (v3f &vertex : vertices) {
 		switch (wall) {
 			case DWM_YP:
-				vertex.rotateXZBy(-45); break;
+				vertex.Y += -size + BS/2;
+				vertex.rotateXZBy(-45);
+				break;
 			case DWM_YN:
-				vertex.rotateXZBy( 45); break;
+				vertex.Y += size - BS/2;
+				vertex.rotateXZBy(45);
+				break;
 			case DWM_XP:
-				vertex.rotateXZBy(  0); break;
+				vertex.X += -size + BS/2;
+				break;
 			case DWM_XN:
-				vertex.rotateXZBy(180); break;
+				vertex.X += -size + BS/2;
+				vertex.rotateXZBy(180);
+				break;
 			case DWM_ZP:
-				vertex.rotateXZBy( 90); break;
+				vertex.X += -size + BS/2;
+				vertex.rotateXZBy(90);
+				break;
 			case DWM_ZN:
-				vertex.rotateXZBy(-90); break;
+				vertex.X += -size + BS/2;
+				vertex.rotateXZBy(-90);
 		}
 	}
 	drawQuad(vertices);
