@@ -122,6 +122,7 @@ enum ItemRotationKind
 	IT_ROT_SELECTED,
 	IT_ROT_HOVERED,
 	IT_ROT_DRAGGED,
+	IT_ROT_OTHER,
 	IT_ROT_NONE, // Must be last, also serves as number
 };
 
@@ -133,4 +134,15 @@ void drawItemStack(video::IVideoDriver *driver,
 		Client *client,
 		ItemRotationKind rotation_kind);
 
+void drawItemStack(
+		video::IVideoDriver *driver,
+		gui::IGUIFont *font,
+		const ItemStack &item,
+		const core::rect<s32> &rect,
+		const core::rect<s32> *clip,
+		Client *client,
+		ItemRotationKind rotation_kind,
+		const v3s16 &angle,
+		const v3s16 &rotation_speed);
+
 #endif
