@@ -230,7 +230,7 @@ private:
 
 	int m_move_id = -1;
 	bool m_move_has_really_moved = false;
-	s64 m_move_downtime = 0;
+	u64 m_move_downtime = 0;
 	bool m_move_sent_as_mouse_event = false;
 	v2s32 m_move_downlocation = v2s32(-10000, -10000);
 
@@ -296,7 +296,7 @@ private:
 	// doubleclick detection variables
 	struct key_event
 	{
-		unsigned int down_time;
+		u64 down_time;
 		s32 x;
 		s32 y;
 	};
