@@ -27,6 +27,7 @@ with this program; if not, write to the Free Software Foundation, Inc.,
 #include <json/json.h>
 #include "cpp_api/s_security.h"
 #include "porting.h"
+#include "convert_json.h"
 #include "debug.h"
 #include "log.h"
 #include "tool.h"
@@ -58,7 +59,7 @@ int ModApiUtil::l_log(lua_State *L)
 		std::string name = luaL_checkstring(L, 1);
 		text = luaL_checkstring(L, 2);
 		if (name == "deprecated") {
-			log_deprecated(L, text);
+			log_deprecated(L, text, 2);
 			return 0;
 		}
 		level = Logger::stringToLevel(name);
@@ -97,12 +98,14 @@ int ModApiUtil::l_parse_json(lua_State *L)
 	Json::Value root;
 
 	{
-		Json::Reader reader;
 		std::istringstream stream(jsonstr);
 
-		if (!reader.parse(stream, root)) {
-			errorstream << "Failed to parse json data "
-				<< reader.getFormattedErrorMessages();
+		Json::CharReaderBuilder builder;
+		builder.settings_["collectComments"] = false;
+		std::string errs;
+
+		if (!Json::parseFromStream(builder, stream, &root, &errs)) {
+			errorstream << "Failed to parse json data " << errs << std::endl;
 			size_t jlen = strlen(jsonstr);
 			if (jlen > 100) {
 				errorstream << "Data (" << jlen
@@ -132,7 +135,7 @@ int ModApiUtil::l_write_json(lua_State *L)
 
 	bool styled = false;
 	if (!lua_isnone(L, 2)) {
-		styled = lua_toboolean(L, 2);
+		styled = readParam<bool>(L, 2);
 		lua_pop(L, 1);
 	}
 
@@ -147,28 +150,22 @@ int ModApiUtil::l_write_json(lua_State *L)
 
 	std::string out;
 	if (styled) {
-		Json::StyledWriter writer;
-		out = writer.write(root);
+		out = root.toStyledString();
 	} else {
-		Json::FastWriter writer;
-		out = writer.write(root);
+		out = fastWriteJson(root);
 	}
 	lua_pushlstring(L, out.c_str(), out.size());
 	return 1;
 }
 
-// get_dig_params(groups, tool_capabilities[, time_from_last_punch])
+// get_dig_params(groups, tool_capabilities)
 int ModApiUtil::l_get_dig_params(lua_State *L)
 {
 	NO_MAP_LOCK_REQUIRED;
 	ItemGroupList groups;
 	read_groups(L, 1, groups);
 	ToolCapabilities tp = read_tool_capabilities(L, 2);
-	if(lua_isnoneornil(L, 3))
-		push_dig_params(L, getDigParams(groups, &tp));
-	else
-		push_dig_params(L, getDigParams(groups, &tp,
-					luaL_checknumber(L, 3)));
+	push_dig_params(L, getDigParams(groups, &tp));
 	return 1;
 }
 
@@ -176,14 +173,13 @@ int ModApiUtil::l_get_dig_params(lua_State *L)
 int ModApiUtil::l_get_hit_params(lua_State *L)
 {
 	NO_MAP_LOCK_REQUIRED;
-	UNORDERED_MAP<std::string, int> groups;
+	std::unordered_map<std::string, int> groups;
 	read_groups(L, 1, groups);
 	ToolCapabilities tp = read_tool_capabilities(L, 2);
 	if(lua_isnoneornil(L, 3))
 		push_hit_params(L, getHitParams(groups, &tp));
 	else
-		push_hit_params(L, getHitParams(groups, &tp,
-					luaL_checknumber(L, 3)));
+		push_hit_params(L, getHitParams(groups, &tp, readParam<float>(L, 3)));
 	return 1;
 }
 
@@ -235,7 +231,7 @@ int ModApiUtil::l_is_yes(lua_State *L)
 	lua_getglobal(L, "tostring"); // function to be called
 	lua_pushvalue(L, 1); // 1st argument
 	lua_call(L, 1, 1); // execute function
-	std::string str(lua_tostring(L, -1)); // get result
+	std::string str = readParam<std::string>(L, -1); // get result
 	lua_pop(L, 1);
 
 	bool yes = is_yes(str);
@@ -243,12 +239,21 @@ int ModApiUtil::l_is_yes(lua_State *L)
 	return 1;
 }
 
+// is_nan(arg)
+int ModApiUtil::l_is_nan(lua_State *L)
+{
+	NO_MAP_LOCK_REQUIRED;
+
+	lua_pushboolean(L, isNaN(L, 1));
+	return 1;
+}
+
 // get_builtin_path()
 int ModApiUtil::l_get_builtin_path(lua_State *L)
 {
 	NO_MAP_LOCK_REQUIRED;
 
-	std::string path = porting::path_share + DIR_DELIM + "builtin";
+	std::string path = porting::path_share + DIR_DELIM + "builtin" + DIR_DELIM;
 	lua_pushstring(L, path.c_str());
 
 	return 1;
@@ -264,7 +269,7 @@ int ModApiUtil::l_compress(lua_State *L)
 
 	int level = -1;
 	if (!lua_isnone(L, 3) && !lua_isnil(L, 3))
-		level = luaL_checknumber(L, 3);
+		level = readParam<float>(L, 3);
 
 	std::ostringstream os;
 	compressZlib(std::string(data, size), os, level);
@@ -337,7 +342,7 @@ int ModApiUtil::l_get_dir_list(lua_State *L)
 	NO_MAP_LOCK_REQUIRED;
 	const char *path = luaL_checkstring(L, 1);
 	bool list_all = !lua_isboolean(L, 2); // if its not a boolean list all
-	bool list_dirs = lua_toboolean(L, 2); // true: list dirs, false: list files
+	bool list_dirs = readParam<bool>(L, 2); // true: list dirs, false: list files
 
 	CHECK_SECURE_PATH(L, path, false);
 
@@ -346,9 +351,9 @@ int ModApiUtil::l_get_dir_list(lua_State *L)
 	int index = 0;
 	lua_newtable(L);
 
-	for (size_t i = 0; i < list.size(); i++) {
-		if (list_all || list_dirs == list[i].dir) {
-			lua_pushstring(L, list[i].name.c_str());
+	for (const fs::DirListNode &dln : list) {
+		if (list_all || list_dirs == dln.dir) {
+			lua_pushstring(L, dln.name.c_str());
 			lua_rawseti(L, -2, ++index);
 		}
 	}
@@ -405,7 +410,7 @@ int ModApiUtil::l_request_insecure_environment(lua_State *L)
 	}
 
 	// Check secure.trusted_mods
-	const char *mod_name = lua_tostring(L, -1);
+	std::string mod_name = readParam<std::string>(L, -1);
 	std::string trusted_mods = g_settings->get("secure.trusted_mods");
 	trusted_mods.erase(std::remove_if(trusted_mods.begin(),
 			trusted_mods.end(), static_cast<int(*)(int)>(&std::isspace)),
@@ -433,7 +438,7 @@ int ModApiUtil::l_get_version(lua_State *L)
 	lua_pushstring(L, g_version_string);
 	lua_setfield(L, table, "string");
 
-	if (strcmp(g_version_string, g_version_hash)) {
+	if (strcmp(g_version_string, g_version_hash) != 0) {
 		lua_pushstring(L, g_version_hash);
 		lua_setfield(L, table, "hash");
 	}
@@ -446,7 +451,7 @@ int ModApiUtil::l_sha1(lua_State *L)
 	NO_MAP_LOCK_REQUIRED;
 	size_t size;
 	const char *data = luaL_checklstring(L, 1, &size);
-	bool hex = !lua_isboolean(L, 2) || !lua_toboolean(L, 2);
+	bool hex = !lua_isboolean(L, 2) || !readParam<bool>(L, 2);
 
 	// Compute actual checksum of data
 	std::string data_sha1;
@@ -484,6 +489,7 @@ void ModApiUtil::Initialize(lua_State *L, int top)
 	API_FCT(get_password_hash);
 
 	API_FCT(is_yes);
+	API_FCT(is_nan);
 
 	API_FCT(get_builtin_path);
 
@@ -516,8 +522,7 @@ void ModApiUtil::InitializeClient(lua_State *L, int top)
 	API_FCT(write_json);
 
 	API_FCT(is_yes);
-
-	API_FCT(get_builtin_path);
+	API_FCT(is_nan);
 
 	API_FCT(compress);
 	API_FCT(decompress);
